import sys
import os
import logging
import time
import sqlite3
from scrapy import Spider, Request
from scrapy.http import Response
from model import Model

logger = logging.getLogger(__name__)

UA = 'Mozilla/5.0 (X11; Linux x86_64; rv:139.0) Gecko/20100101 Firefox/139.0'

def yield_from_joinmbz():
    # load from jmbz api
    import requests
    url = 'https://instances.joinmobilizon.org/api/v1/instances?start=0&count=1000'
    joinres = requests.get(url, headers={'user-agent': UA}).json().get('data')
    # load from db
    with sqlite3.connect(f'{os.environ.get("DATADIR")}/sqlpage.db', timeout=15.0, isolation_level='IMMEDIATE') as db:
        rows = db.execute('select rowid, domain from instances').fetchall()
        max_rowid = max(row[0] for row in rows)
        domains = {row[1] for row in rows}
        for r in joinres:
            if r['host'] not in domains:
                logging.debug(f'''new: {r['id']}, {r['host']}''')
                # insert new records
                db.execute('''insert into instances(rowid, domain, failure, "createdAt") values (?,?,?,?)''', (
                   r['id']
                 , r['host']
                 , -1
                 , int(time.time())
                ))
    yield from yield_from_db()


def yield_from_db():
    with sqlite3.connect(f'file:{os.environ.get("DATADIR")}/sqlpage.db?mode=ro', uri=True) as db:
        for row in db.execute('select rowid, domain from instances where failure < 5'):
            yield row

def yield_tests():
    yield (105, 'keskonfai.fr',)
    yield (7, 'mobilizon.fr',)

SOURCES = {
    "test": yield_tests,
    "local": yield_from_db,
    "remote": yield_from_joinmbz,
}

class Fedator(Spider):
    name = "fedator"
    custom_settings = {
        "USER_AGENT": UA,
        "ROBOTSTXT_OBEY": False,
        "REFERER_ENABLED": False,
        "COOKIES_ENABLED": False,
        "TELNETCONSOLE_ENABLED": False,
        "HTTPCACHE_ENABLED": False,
        "DEFAULT_REQUEST_HEADERS": {
            "Accept": "application/json",
            "Content-Type": "application/json",
        },
        "DOWNLOAD_TIMEOUT": 10,
    }


    async def start(self):
        gql_query = open('src/query.gql', 'r').read()
        from json import dumps
        body = dumps({"query": gql_query})
        source = SOURCES.get(os.environ.get("SOURCE","test"))
        for row in source():
            domain = row[1]
            yield Request(
                url = f"https://{domain}/api",
                method = 'POST',
                body = body,
                meta = { "record": row, },
            )

    def parse(self, response: Response):
        res:Model = Model.model_validate_json(response.body)
        meta = response.request.meta
        instance_id = meta.get('record')[0]
        with sqlite3.connect(f'{os.environ.get("DATADIR")}/sqlpage.db', timeout=15.0, isolation_level='IMMEDIATE') as db:
            # insert new stats
            stats = res.data.statistics
            db.execute('''insert into stats(insertedAt, instance_id, users, local_groups, total_groups, local_events, total_events, local_comments, total_comments, following, followers) values (?,?,?,?,?,?,?,?,?,?,?)
                ''', (
                   int(time.time())  # insertedAt
                 , instance_id
                 , stats.number_of_users # users
                 , stats.number_of_local_groups
                 , stats.number_of_groups
                 , stats.number_of_local_events
                 , stats.number_of_events
                 , stats.number_of_local_comments
                 , stats.number_of_comments
                 , stats.number_of_instance_followings
                 , stats.number_of_instance_followers
                    
                ))
            # update info
            config = res.data.config
            db.execute('''update instances set name=?,slogan=?,description=?,languages=?,open=?,version=?
                -- ,location=?
                ,failure=?,updatedAt=? where rowid=?                 
                ''', (
                      config.name
                    , config.slogan
                    , config.description
                    , ",".join(config.languages)
                    , config.registrations_open
                    , config.version
                    #, config.location
                    , 0  # failure
                    , int(time.time())  # updatedAt
                    , instance_id
                ))
        
