from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel, Field


class Config(BaseModel):
    country_code: Any = Field(..., alias='countryCode')
    description: Optional[str] = None
    languages: List[str]
    long_description: Optional[str] = Field(..., alias='longDescription')
    name: str
    registrations_open: bool = Field(..., alias='registrationsOpen')
    slogan: Optional[str] = None
    version: str


class Statistics(BaseModel):
    number_of_comments: int = Field(..., alias='numberOfComments')
    number_of_events: int = Field(..., alias='numberOfEvents')
    number_of_groups: int = Field(..., alias='numberOfGroups')
    number_of_instance_followers: Optional[int] = Field(..., alias='numberOfInstanceFollowers')
    number_of_instance_followings: Optional[int] = Field(..., alias='numberOfInstanceFollowings')
    number_of_local_comments: int = Field(..., alias='numberOfLocalComments')
    number_of_local_events: int = Field(..., alias='numberOfLocalEvents')
    number_of_local_groups: int = Field(..., alias='numberOfLocalGroups')
    number_of_users: int = Field(..., alias='numberOfUsers')


class Data(BaseModel):
    config: Config
    statistics: Statistics


class Model(BaseModel):
    data: Data

